@@ -624,7 +624,7 @@ str_vformat(
     sp = vbuff->curpos;
     bep = vbuff->endpos;
 
-    while (*fmt) {
+    while (*fmt != NUL) {
         if (*fmt != '%') {
             INS_CHAR(*fmt, sp, bep, cc);
         }
@@ -1033,16 +1033,14 @@ static int str_flush_real(str_vformat_t *out_handle)
     return -1;
 }
 
-int str_format(char *s, str_size_t n, const char *fmt, ...)
+int str_format_va(char *s, str_size_t n, const char *fmt, va_list ap)
 {
-    va_list ap;
     str_vformat_t handle;
     char buf[STR_FORMAT_BUFLEN];
     int rv;
 
     if (n == 0)
         return 0;
-    va_start(ap, fmt);
     if (s == NULL) {
         /* fake formatting, i.e., calculate output length only */
         handle.curpos    = buf;
@@ -1067,7 +1065,16 @@ int str_format(char *s, str_size_t n, const char *fmt, ...)
         if (rv == -1)
             rv = n;
     }
-    va_end(ap);
     return rv;
 }
 
+int str_format(char *s, str_size_t n, const char *fmt, ...)
+{
+    va_list ap;
+
+    va_start(ap, fmt);
+    str_format_va(s, n, fmt, ap);
+    va_end(ap);
+    return 0;
+}
+
