@@ -257,6 +257,16 @@ str_parse_format(
 
 /* the API parsing function */
 int str_parse(const char *string, const char *pattern, ...)
+{
+    va_list ap;
+    int rv;
+
+    va_start(ap, pattern);
+    rv = str_parse_va(string, pattern, ap);
+    va_end(ap);
+    return rv;
+}
+int str_parse_va(const char *string, const char *pattern, va_list ap)
 {
     pcre *p_pcre;
     pcre_extra *p_pcre_extra;
@@ -278,7 +288,6 @@ int str_parse(const char *string, const char *pattern, ...)
     int i;
     int k;
     int l;
-    va_list ap;
     int ismop;
     int issop;
     char buf[128];
@@ -433,7 +442,6 @@ int str_parse(const char *string, const char *pattern, ...)
     /*
      * Create either matching or substitution result
      */
-    va_start(ap, pattern);
     if (ismop && cap_num > 0) {
         /* 
          * extract captured substrings into caller provided pointer variables 
@@ -568,12 +576,12 @@ int str_parse(const char *string, const char *pattern, ...)
         sf.data[4].p = cap_vec;
         sf.data[5].i = cap_num;
         str_vformat(&sf, buf_ptr, ap);
+        *((*cpp)+l) = NUL;
 
         /* free temp buffer */
         if (buf_ptr != buf)
             free(buf_ptr);
     }
-    va_end(ap);
 
     /* cleanup */
     if (cap_vec != NULL)
