@@ -69,7 +69,7 @@ char *str_span(const char *s, str_size_t n, const char *charset, int mode)
             sp--;
         l2:
         sc = *sp--;
-        if (sp >= (s-1)) {
+        if (sp >= s) {
             for (cp = charset; (cc = *cp++) != NUL; )
                 if (sc == cc)
                     goto l2;
@@ -99,7 +99,7 @@ char *str_span(const char *s, str_size_t n, const char *charset, int mode)
             sp--;
         l4a:
         sc = *sp--;
-        if (sp >= (s-1)) {
+        if (sp >= s) {
             for (cp = charset; (cc = *cp++) != NUL; )
                 if (sc == cc)
                     goto l4;
@@ -135,7 +135,6 @@ char *str_locate(const char *ay, str_size_t n, const char *ax)
     register const unsigned char *tx;
     register const unsigned char *ty;
     int tm;
-    int nulterm;
 
     /*
      * Consistency checks
@@ -150,9 +149,7 @@ char *str_locate(const char *ay, str_size_t n, const char *ax)
      */
 
     /* length of y [inlined n = strlen(y)] */
-    nulterm = TRUE;
-    if (n > 0) {
-        nulterm = FALSE;
+    if (n == 0) {
         ty = y;
         while (*ty++ != NUL)
             /*nop*/;
@@ -166,7 +163,7 @@ char *str_locate(const char *ay, str_size_t n, const char *ax)
     m = (tx-x-1);
 
     /* fill whole bad char skip array */
-    for (i = 0; i < sizeof(qs_bc); i++) 
+    for (i = 0; i < (sizeof(qs_bc)/sizeof(int)); i++) 
         qs_bc[(int)i] = m+1;
 
     /* override positions related to pattern */
@@ -177,42 +174,24 @@ char *str_locate(const char *ay, str_size_t n, const char *ax)
      * Searching
      */
 
-    if (nulterm) {
-        /* search inside a nul-terminated string */
-        while ((i = *(y+m)) != NUL) {
-            /* compare first char (speed-up) */
-            if (*y == *x) {
-                /* compare remaining chars */
-                ty = y+1;
-                tx = x+1;
-                tm = m-1;
-                do {
-                    if (tm-- == 0)
-                        return (char *)y; /* found pattern */
-                } while (*ty++ == *tx++);
-            }
-            y += qs_bc[i]; /* perform shift */
-        }
-    }
-    else {
-        /* search inside a len-terminated string */
-        while (n >= m) {
-            /* compare first char (speed-up) */
-            if (*y == *x) {
-                /* compare remaining chars */
-                ty = y+1;
-                tx = x+1;
-                tm = m-1;
-                do {
-                    if (tm-- == 0)
-                        return (char *)y; /* found pattern */
-                } while (*ty++ == *tx++);
-            }
-            i = qs_bc[(int)*(y+m)];  /* perform shift */
-            y += i;
-            n -= i;
+    /* search inside a len-terminated string */
+    while (n >= m) {
+        /* compare first char (speed-up) */
+        if (*y == *x) {
+            /* compare remaining chars */
+            ty = y+1;
+            tx = x+1;
+            tm = m-1;
+            do {
+                if (tm-- == 0)
+                    return (char *)y; /* found pattern */
+            } while (*ty++ == *tx++);
         }
+        i = qs_bc[(int)*(y+m)];  /* perform shift */
+        y += i;
+        n -= i;
     }
+
     return NULL;
 }
 
