@@ -34,6 +34,88 @@
 
 #define LONG_STRING 1024
 
+/*
+ * Test String Locating
+ */
+
+struct {
+    char *s; str_size_t n; char *p; int o;
+} loctab[] = {
+    { "foo bar quux", 0, "", 0 },
+    { "foo bar quux", 0, "foo", 0 },
+    { "foo bar quux", 0, "bar", 4 },
+    { "foo bar quux", 0, "quux", 8 },
+    { "foo bar quux", 0, "x", 11 },
+    { "foo bar quux", 3, "foo", 0 },
+    { "foo bar quux", 4, "x", -1 },
+    { "foo bar quux", 5, "bar", -1 },
+    { "foo bar", 0, "foo bar", 0 },
+    { "foo bar", 0, "foo bar quux", -1 },
+    { NULL, 0, NULL, 0 }
+};
+
+static void test_locate(int *ok, int *fail)
+{
+    int i;
+    char *rv;
+
+    for (i = 0; loctab[i].s != NULL; i++) {
+        rv = str_locate(loctab[i].s, loctab[i].n, loctab[i].p);
+        printf("str_locate(\"%s\", %d, \"%s\") = \"%s\"\n",
+                loctab[i].s, loctab[i].n, loctab[i].p, rv == NULL ? "[NULL]" : rv);
+        if ((rv-loctab[i].s == loctab[i].o) || (rv == NULL && loctab[i].o == -1))
+            (*ok)++;
+        else {
+            fprintf(stderr, "ERROR: result was \"%s\", expected \"%s\"\n",
+                    rv, loctab[i].s+loctab[i].o);
+            (*fail)++;
+        }
+    }
+    return;
+}
+
+/*
+ * Test String Spanning
+ */
+
+struct {
+    char *s; str_size_t n; char *cs; int m; int o;
+} spantab[] = {
+    { "foo bar quux", 0, "", 0, 0 },
+    { "foo bar quux", 0, "xyz", 0, 0 },
+    { "foo bar quux", 0, "fo ", 0, 4 },
+    { "foo bar quux", 0, "b", STR_COMPLEMENT, 4 },
+    { "foo bar quux", 0, "", STR_COMPLEMENT, 12 },
+    { "foo bar quux", 0, "", STR_RIGHT, 11 },
+    { "foo bar quux", 0, "abc", STR_RIGHT, 11 },
+    { "foo bar quux", 0, "qux ", STR_RIGHT, 6 },
+    { "foo bar quux", 0, "r", STR_RIGHT|STR_COMPLEMENT, 6 },
+    { "foo bar quux", 0, "", STR_RIGHT|STR_COMPLEMENT, 0 },
+    { "", 0, "", STR_COMPLEMENT, 0 },
+    { "", 0, "", STR_RIGHT|STR_COMPLEMENT, 0 },
+    { NULL, 0, NULL, 0, 0 }
+};
+
+static void test_span(int *ok, int *fail)
+{
+    int i;
+    char *rv;
+
+    for (i = 0; spantab[i].s != NULL; i++) {
+        rv = str_span(spantab[i].s, spantab[i].n, spantab[i].cs, spantab[i].m);
+        printf("str_span(\"%s\", %d, \"%s\", %d) = \"%s\"\n",
+                spantab[i].s, spantab[i].n, spantab[i].cs, spantab[i].m, rv);
+        if (rv-spantab[i].s == spantab[i].o)
+            (*ok)++;
+        else {
+            fprintf(stderr, "ERROR: result was \"%s\", expected \"%s\"\n",
+                    rv, spantab[i].s+spantab[i].o);
+            (*fail)++;
+        }
+    }
+    return;
+}
+
 /*
  * Test String Tokenization
  */
@@ -350,6 +432,8 @@ struct {
     int ok;
     int fail;
 } tests[] = {
+    { "Locating",    test_locate,      0, 0 },
+    { "Spanning",    test_span,        0, 0 },
     { "Tokenizing",  test_tokenize,    0, 0 },
     { "Parsing",     test_parsing,     0, 0 },
     { "Formatting",  test_formatting,  0, 0 },
